/******************************************************************/
/*** ColorLib - Color graphic library for fx-CG 20 - fx-CG 10	***/
/*** ColorLib is a free software								***/
/***															***/
/*** @author Lancelot											***/
/***															***/
/*** @file ColorLib.c											***/
/*** Include header for ColorLib								***/
/***															***/
/*** @date 27 October 2013										***/
/*** version 1.000												***/
/******************************************************************/

 
#ifndef COLORLIB
#define COLORLIB


/****************************************************/
/** uncomment #define of functions you want to use **/
/****************************************************/

#define CL_ALL //Auto define all functions

/** graphic functions: **/
// #define CL_PIXEL
// #define CL_PIXEL_ASM 
// #define CL_POINT_ALPHA
// #define CL_GET_PIXEL
// #define CL_LINE
// #define CL_VERTICAL_LINE
// #define CL_HORIZONTAL_LINE
// #define CL_FILLVRAM
// #define CL_CIRCLE
// #define CL_DISCUS
// #define CL_FILLED_RECTANGLE
// #define CL_POLY
// #define CL_FILLED_POLYGON

/** display sprite functions: **/
// #define CL_SPRITE
// #define CL_ALPHASPRITE
// #define CL_SPRITE16

/** text functions: **/
// #define CL_TEXT


/**************************/
/** Useful Constants     **/
/**************************/

#define LCD_WIDTH_PX 384
#define LCD_HEIGHT_PX 216
#define MONOSPACE    0x00000001
#define ANTIALIASING 0x00000002
#define SCREEN_WIDTH 384

/**************************/
/** Flags                **/
/**************************/

// Font flags :
#define abs(a) ((a) < 0 ? -(a) : (a))
#define min(a,b) (((a) < (b))? (a) : (b))
#define max(a,b) (((a) > (b))? (a) : (b))
#define sgn(a) ((a) < 0 ? (-1) : (a) > 0 ? (1) : (a))

/**************************/
/** Structures           **/
/**************************/

typedef struct {
	char* data;
	int width, height, flags;
} Font;

/**************************/
/** Functions prototypes **/
/**************************/

void* GetVRAMAdress();


// graphic functions prototypes:
void CL_Pixel(int x, int y, unsigned short color);short CL_Getpixel(short x, short y);

void CL_Pixel_asm(unsigned short x, unsigned short y, unsigned short color);

short CL_Getpixel(short x, short y);

void CL_Line(int xi, int yi, int xf, int yf, unsigned short color);

void CL_Vertical_Line(int x, int y1, int y2, unsigned short color);

void CL_Horizontal_Line(int x1, int x2, int y, unsigned short color);

void CL_FillVRAM(short color);

void CL_Circle(int x0, int y0, int rayon, int couleur);

void CL_Discus(int x0, int y0, int rayon, int couleur);

void CL_Filled_Rectangle( int xi, int yi, int xi2, int yi2, unsigned short color);

void CL_Poly(int xy[], int points, int color);

int CL_Filled_polygon_quicksord_partition(int *t, int p, int r);

void CL_Filled_polygon_quicksord(int* t, int p, int r);

void CL_Filled_polygon(const int *x, const int *y, int nb_vertices, unsigned short color);


// display sprite functions:
void CL_bmp_masked_nbit(const unsigned char* data, int x, int y, int width, int height, const short* palette, short maskColor, unsigned int bitwidth);

void CL_bmp_alpha(int x, int y, int width, int height, short* bitmap, short alpha);

void CL_bmp_masked(short* bitmap, int x, int y, int width, int height, short mask);


// text functions prototypes:
void text_setFont(Font* font);

Font* text_getActualFont();

void CL_text_printC(int x, int y, char c, int size, int color);

void CL_text_print(int x, int y, char* c, int size, int color);

int text_widthC(char c);

int text_width(char* c);

#endif // COLORLIB

